import os

from frigg_settings.helpers import FileSystemWrapper


def path(*args):
    return os.path.join(os.path.dirname(os.path.dirname(__file__)), *args)


def test_filesystemwrapper_list_files():
    wrapper = FileSystemWrapper()
    files = wrapper.list_files(path())

    # This check cannot check the exact files because of
    # generated coverage files.
    assert '.frigg.yml' in files
    assert '.gitignore' in files
    assert 'frigg_settings' not in files
    assert 'tests' not in files


def test_filesystemwrapper_read_file():
    wrapper = FileSystemWrapper()
    assert(
        wrapper.read_file(path('MANIFEST.in')) ==
        'include setup.py README.md MANIFEST.in LICENSE\n'
    )


def test_filesystemwrapper_file_exist():
    wrapper = FileSystemWrapper()
    assert wrapper.file_exist(path('setup.py'))
    assert not wrapper.file_exist(path('non-exsting'))
    assert not wrapper.file_exist(path('tests'))



import os

import pytest
import yaml

from frigg_settings.helpers import FileSystemWrapper
from frigg_settings.settings import build_settings


def load_fixture(filename):
    path = os.path.join(
        os.path.dirname(__file__),
        'fixtures',
        filename
    )
    with open(path) as f:
        return yaml.load(f.read())


@pytest.fixture
def v1():
    return load_fixture('v1.yaml')


@pytest.fixture
def v2():
    return load_fixture('v2.yaml')


def test_should_load_v1(mocker, v1):
    mocker.patch('frigg_settings.settings.yaml.load', return_value=v1)
    mocker.patch('frigg_settings.settings.get_path_of_settings_file')
    settings = build_settings('.', FileSystemWrapper())

    assert settings.tasks is not None
    assert settings.tasks['setup'] is not None
    assert settings.tasks['tests'] is not None
    assert settings.tasks['verbose'] is not None

    assert len(settings.tasks['setup']) == 1
    assert len(settings.tasks['tests']) == 3
    assert len(settings.tasks['verbose']) == 1

    assert settings.coverage['path'] == 'c.xml'
    assert settings.coverage['parser'] == 'cobertura'

    assert settings.preview['tasks'] is not None
    assert len(settings.preview['tasks']) == 4


def test_should_load_v2(mocker, v2):
    mocker.patch('frigg_settings.settings.yaml.load', return_value=v2)
    mocker.patch('frigg_settings.settings.get_path_of_settings_file')
    settings = build_settings('.', FileSystemWrapper())

    assert settings.tasks is not None
    assert settings.tasks['setup'] is not None
    assert settings.tasks['tests'] is not None
    assert settings.tasks['verbose'] is not None

    assert len(settings.tasks['setup']) == 1
    assert len(settings.tasks['tests']) == 3
    assert len(settings.tasks['verbose']) == 1

    assert settings.coverage['path'] == 'c.xml'
    assert settings.coverage['parser'] == 'cobertura'

    assert settings.preview['tasks'] is not None
    assert len(settings.preview['tasks']) == 4



__version__ = '3.1.2'

from frigg_settings.model import FriggSettings  # noqa
from frigg_settings.helpers import FileSystemWrapper  # noqa
from frigg_settings.settings import build_tasks, load_settings_file, build_settings  # noqa




class FriggSettings(object):

    def __init__(self, obj=None):
        self.tasks = {
            'setup': [],
            'tests': [],
            'verbose': [],
            'after_success': [],
            'after_failure': [],
        }
        self.webhooks = []
        self.services = []
        self.preview = {}
        self.coverage = None

        if obj is None:
            return

        if 'tasks' in obj and isinstance(obj['tasks'], list):
            obj = self.convert_v1_to_v2(obj)

        self.update(obj)

    def update(self, obj):
        tasks = self.tasks
        self.__dict__.update(obj)
        if 'tasks' in obj:
            tasks.update(obj['tasks'])

        self.tasks = tasks

    def validate(self):
        return True

    @staticmethod
    def convert_v1_to_v2(content):
        tasks = {'setup': [], 'tests': [], 'verbose': []}

        if 'setup_tasks' in content:
            tasks['setup'] = content['setup_tasks']

        if 'tasks' in content:
            tasks['tests'] = content['tasks']

        if 'verbose_tasks' in content:
            tasks['verbose'] = content['verbose_tasks']

        content.update({'tasks': tasks})
        return content

    @property
    def has_tests_tasks(self):
        return len(self.tasks['tests']) > 0

    def has_after_tasks(self, status):
        if status:
            return len(self.tasks['after_success']) > 0
        return len(self.tasks['after_failure']) > 0



import os

import pytest

from frigg_settings import build_settings, build_tasks, load_settings_file
from frigg_settings.helpers import FileSystemWrapper


@pytest.fixture
def settings_file():
    return ('tasks:\n - tox -e py34\n - tox -e flake8\n - tox -e isort\n - coverage report -m '
            '&& coverage xml\n\ncoverage:\n  path: coverage.xml\n  parser: python\n')


@pytest.fixture
def settings_dict():
    return {'tasks': {'tests': ['tox']}, 'coverage': None}


@pytest.fixture
def runner():
    return FileSystemWrapper()


def test_build_tasks(mocker, runner):
    mock_list_dir = mocker.patch('frigg_settings.helpers.FileSystemWrapper.list_files')
    build_tasks('path', runner)
    mock_list_dir.assert_called_once_with('path')


def test_load_settings_file(mocker, runner, settings_file):
    mock_read_file = mocker.patch('frigg_settings.helpers.FileSystemWrapper.read_file',
                                  return_value=settings_file)
    load_settings_file('.frigg.yml', runner)
    mock_read_file.assert_called_once_with('.frigg.yml')


def test_load_settings_file_no_path(runner):
    assert load_settings_file(None, runner) is None


def test_build_settings(mocker, runner, settings_dict):
    mocker.patch('frigg_settings.settings.detect_tox_environments',
                 return_value=['tests', 'flake8'])
    mock_load_settings_file = mocker.patch('frigg_settings.settings.load_settings_file',
                                           return_value=settings_dict)
    build_settings(os.path.dirname(os.path.dirname(__file__)), runner)

    assert mock_load_settings_file.call_args_list[0].endswith('frigg/frigg-settings/.frigg.yml')


def test_build_settings_should_load_tox_tasks(mocker, runner, settings_dict):
    mocker.patch('frigg_settings.settings.detect_tox_environments',
                 return_value=['tests', 'flake8'])
    mocker.patch('frigg_settings.settings.load_settings_file', return_value=settings_dict)
    settings = build_settings(os.path.dirname(os.path.dirname(__file__)), runner)

    assert 'tox -e tests' in settings.tasks['tests']
    assert 'tox -e flake8' in settings.tasks['tests']
    assert 'tox' not in settings.tasks['tests']



from frigg_settings import FriggSettings


def test_settings_should_have_defaults():
    settings = FriggSettings()
    assert settings.tasks is not None
    assert settings.services is not None
    assert settings.preview is not None


def test_update_should_update_dict():
    settings = FriggSettings()
    settings.update({'tasks': {'tests': ['tox']}})
    assert settings.tasks['tests'] == ['tox']


def test_update_should_keep_defaults_of_untouched_fields():
    settings = FriggSettings()
    settings.update({'tasks': {'tests': ['tox']}})
    assert settings.tasks['tests'] == ['tox']
    assert settings.tasks['setup'] == []
    assert settings.tasks['verbose'] == []
    assert settings.tasks['after_success'] == []
    assert settings.tasks['after_failure'] == []


def test_has_test_tasks_should_return_correct_value():
    assert FriggSettings({'tasks': {'tests': ['tox']}}).has_tests_tasks
    assert not FriggSettings({'tasks': {'tests': []}}).has_tests_tasks


def test_has_after_tasks_should_return_correct_value_for_successful_builds():
    assert FriggSettings({'tasks': {'after_success': ['tox']}}).has_after_tasks(True)
    assert not FriggSettings({'tasks': {'after_success': []}}).has_after_tasks(True)


def test_has_after_tasks_should_return_correct_value_for_unsuccessful_builds():
    assert FriggSettings({'tasks': {'after_failure': ['tox']}}).has_after_tasks(False)
    assert not FriggSettings({'tasks': {'after_failure': []}}).has_after_tasks(False)


def test_model_should_create_new_objects():
    FriggSettings({'tasks': {'tests': ['tox']}})
    assert FriggSettings({'tasks': {'setup': ['tox']}}).tasks['tests'] != ['tox']


def test_model_without_tasks():
    assert 'setup' in FriggSettings({}).tasks
